#!/usr/bin/env python3

# video_chain_oscillator.py
# Copyright (c) 2017 Dai Owaki <owaki@tohoku.ac.jp>
# 
import matplotlib.pyplot as plt
import numpy as np
import matplotlib as mpl
import matplotlib.gridspec as gridspec
import matplotlib.animation as animation
import itertools as it


def video(x, dt, max_t, params):
    
    omega   = params[0]
    o_num   = params[1]
    epsilon = params[2]
    psi     = params[3]
    
    cmap1 = plt.get_cmap("hsv")
    cmap2 = plt.get_cmap("Reds")

    #initial setting of figures
    fig = plt.figure(figsize=(10, 6.))
    gs  = gridspec.GridSpec(2,3)

    # setting for ax1 area
    ax1 = fig.add_subplot(gs[0,0], xlim=(-1, 1), ylim=(-1, 1))
    #ax1.set_xlabel('x[m]')
    #ax1.set_ylabel('y[m]')
    ax1.grid()

    radius = 0.3

    circle1 = plt.Circle((-0.5, 0.5),radius,color='k', fill=False)
    circle2 = plt.Circle((-0.5,-0.5),radius,color='k', fill=False)
    circle3 = plt.Circle(( 0.5, 0.5),radius,color='k', fill=False)
    circle4 = plt.Circle(( 0.5,-0.5),radius,color='k', fill=False)


    osci0, = plt.plot([], [], 'o', c=cmap1(float(o_num-3)/o_num), markersize = 10, animated = True)
    osci1, = plt.plot([], [], 'o', c=cmap1(float(o_num-2)/o_num), markersize = 10, animated = True)
    osci2, = plt.plot([], [], 'o', c=cmap1(float(o_num-1)/o_num), markersize = 10, animated = True)
    osci3, = plt.plot([], [], 'o', c=cmap1(float(o_num)/o_num), markersize = 10, animated = True)

    connect_width = epsilon
    connect_color = psi

    line12, = plt.plot((-0.5,-0.5), (-0.2, 0.2), '-', c=cmap1((np.pi+connect_color[0][1])/(2*np.pi)), lw=10*connect_width[0][1], alpha=1.0, animated = False)
    line13, = plt.plot((-0.2, 0.2), ( 0.5, 0.5), '-', c=cmap1((np.pi+connect_color[0][2])/(2*np.pi)), lw=10*connect_width[0][2], alpha=1.0, animated = False)
    line24, = plt.plot((-0.2, 0.2), (-0.5,-0.5), '-', c=cmap1((np.pi+connect_color[1][3])/(2*np.pi)), lw=10*connect_width[1][3], alpha=1.0, animated = False)
    line34, = plt.plot(( 0.5, 0.5), (-0.2, 0.2), '-', c=cmap1((np.pi+connect_color[2][3])/(2*np.pi)), lw=10*connect_width[2][3], alpha=1.0, animated = False)


    time_template = 'time = %.2f s'
    time_text = ax1.text(0.54, 0.925, '', transform=ax1.transAxes)


    # setting for ax2 area
    ax2 = fig.add_subplot(gs[0,1:3], xlim=(0, max_t), ylim=(-1, 1))
    ax2.set_xlabel('time[s]')
    ax2.set_ylabel('sin(phi_i)')
    ax2.grid()


    time  =  np.arange(0.0, max_t, dt) # make a list fot time

    line0, = plt.plot(time, np.sin(x[:,0]), '-', c=cmap1(float(o_num-3)/o_num), lw=1, alpha=0.5)
    line1, = plt.plot(time, np.sin(x[:,2]), '-', c=cmap1(float(o_num-2)/o_num), lw=1, alpha=0.5)
    line2, = plt.plot(time, np.sin(x[:,4]), '-', c=cmap1(float(o_num-1)/o_num), lw=1, alpha=0.5)
    line3, = plt.plot(time, np.sin(x[:,6]), '-', c=cmap1(float(o_num)/o_num), lw=1, alpha=0.5)


    bar, = plt.plot([], [], 'r-', lw=1, animated=True)           #draw time bar w anime

    posci0, = plt.plot([], [], 'o', c=cmap1(float(o_num-3)/o_num), markersize = 5,  animated=True)
    posci1, = plt.plot([], [], 'o', c=cmap1(float(o_num-2)/o_num), markersize = 5,  animated=True)
    posci2, = plt.plot([], [], 'o', c=cmap1(float(o_num-1)/o_num), markersize = 5,  animated=True)
    posci3, = plt.plot([], [], 'o', c=cmap1(float(o_num)/o_num), markersize = 5,  animated=True)



    # setting for ax2 area
    ax3 = fig.add_subplot(gs[1,1:3], xlim=(0, max_t), ylim=(-2.5*np.pi,2.5*np.pi))
    ax3.set_xlabel('time[s]')
    ax3.set_ylabel('phi_1-phi_2')
    ax3.grid()
    
    dline0, = plt.plot(time, x[:,2]-x[:,0], '-', c=cmap2(float(o_num-3)/(o_num-1)), lw=1, alpha=0.5)
    dline1, = plt.plot(time, x[:,4]-x[:,0], '-', c=cmap2(float(o_num-2)/(o_num-1)), lw=1, alpha=0.5)
    dline2, = plt.plot(time, x[:,6]-x[:,0], '-', c=cmap2(float(o_num-1)/(o_num-1)), lw=1, alpha=0.5)
    
    
    bar2, = plt.plot([], [], 'r-', lw=1, animated=True)           #draw time bar w anime
    dposci0, = plt.plot([], [], 'o', c=cmap2(float(o_num-3)/(o_num-1)), markersize = 5,  animated=True)
    dposci1, = plt.plot([], [], 'o', c=cmap2(float(o_num-2)/(o_num-1)), markersize = 5,  animated=True)
    dposci2, = plt.plot([], [], 'o', c=cmap2(float(o_num-1)/(o_num-1)), markersize = 5,  animated=True)
    

    # setting for ax4 area
    ax4 = fig.add_subplot(gs[1,0])
    #ax4.set_xlabel('theta[rad]')
    #ax4.set_ylabel('dtheta[rad]')
    #ax4.grid()
    ax4.tick_params(labelbottom="off",bottom="off")
    ax4.tick_params(labelleft="off",left="off")
    ax4.set_xticklabels([])
    
    params_text1 = ax4.text(0.1, 0.9, '', transform=ax4.transAxes)
    params_text2 = ax4.text(0.1, 0.8, '', transform=ax4.transAxes)
    params_text3 = ax4.text(0.1, 0.7, '', transform=ax4.transAxes)
    params_text4 = ax4.text(0.1, 0.6, '', transform=ax4.transAxes)
    params_text5 = ax4.text(0.1, 0.5, '', transform=ax4.transAxes)
    params_text6 = ax4.text(0.1, 0.4, '', transform=ax4.transAxes)
    params_text7 = ax4.text(0.1, 0.3, '', transform=ax4.transAxes)
    params_text8 = ax4.text(0.1, 0.2, '', transform=ax4.transAxes)
    params_text9 = ax4.text(0.1, 0.1, '', transform=ax4.transAxes)


    plt.tight_layout()
                      
    #initial function for animation
    def init():
        ax1.add_patch(circle1)
        ax1.add_patch(circle2)
        ax1.add_patch(circle3)
        ax1.add_patch(circle4)

        return circle1, circle2, circle3, circle4, osci0, osci1, osci2, osci3, \
                                                    line0, line1, line2, line3, \
                                                    dline0, dline1, dline2, \
                                                    bar, posci0, posci1, posci2, posci3, time_text, \
                                                    bar2, dposci0, dposci1, dposci2, \
                                                    params_text1, params_text2, params_text3, params_text4, params_text5, \
                                                    params_text6, params_text7, params_text8, params_text9, \
                                                    line12, \


    #function for animation
    def anime(i):
        next_bx   = [time[i], time[i]]  # x positions for time bar
        next_by   = [-1, 1]           # y positions for time bar
        bar.set_data(next_bx, next_by)                              # set time bar position

        osci0.set_data(-0.5+radius*np.cos(x[i,0]),  0.5+radius*np.sin(x[i,0]))
        osci1.set_data(-0.5+radius*np.cos(x[i,2]), -0.5+radius*np.sin(x[i,2]))
        osci2.set_data( 0.5+radius*np.cos(x[i,4]),  0.5+radius*np.sin(x[i,4]))
        osci3.set_data( 0.5+radius*np.cos(x[i,6]), -0.5+radius*np.sin(x[i,6]))


        posci0.set_data(time[i], np.sin(x[i,0]))
        posci1.set_data(time[i], np.sin(x[i,2]))
        posci2.set_data(time[i], np.sin(x[i,4]))
        posci3.set_data(time[i], np.sin(x[i,6]))


        dposci0.set_data(time[i], x[i,2]-x[i,0])
        dposci1.set_data(time[i], x[i,4]-x[i,0])
        dposci2.set_data(time[i], x[i,6]-x[i,0])

        time_text.set_text(time_template % time[i]) # display timer

        params_text1.set_text('omega      = %.4f rad/s' % (params[0])) # display params
        params_text2.set_text('epsilon_12 = %.4f ' % (connect_width[0][1])) # display params
        params_text3.set_text('epsilon_13 = %.4f ' % (connect_width[0][2])) # display params
        params_text4.set_text('epsilon_24 = %.4f ' % (connect_width[1][3])) # display params
        params_text5.set_text('epsilon_34 = %.4f ' % (connect_width[2][3])) # display params
        params_text6.set_text('psi_12 = %.4f rad' % (connect_color[0][1])) # display params
        params_text7.set_text('psi_13 = %.4f rad' % (connect_color[0][2])) # display params
        params_text8.set_text('psi_24 = %.4f rad' % (connect_color[1][3])) # display params
        params_text9.set_text('psi_34 = %.4f rad' % (connect_color[2][3])) # display params


        ax1.add_patch(circle1)
        ax1.add_patch(circle2)
        ax1.add_patch(circle3)
        ax1.add_patch(circle4)

        return circle1, circle2, circle3, circle4, osci0, osci1, osci2, osci3, \
                                                   line0, line1, line2, line3, \
                                                  dline0, dline1, dline2, \
                                             bar, posci0, posci1, posci2, posci3, time_text, \
                                            bar2, dposci0, dposci1, dposci2, \
                                            params_text1, params_text2, params_text3, params_text4, params_text5, \
                                            params_text6, params_text7, params_text8, params_text9, \
                                            line12, \



    ani = animation.FuncAnimation(fig, anime, np.arange(1, len(x)), interval=dt*1.0e+3, blit=True, init_func=init)
    #ani.save('py_quad_oscillators_(R-Gallop).mp4', writer='ffmpeg')

    plt.show()

